describe('ZhLayoutFloatingCardController', function(){

    var widget;
    var scope;
    var zhLayoutPreferences;
    var zhPreferences;
    var zhLayoutConfigController;
    var applicationContext;
    var query;
    var screenService;
    var metaDataFactory;
    var scopeDestroyFunction;

    beforeEach(function(){
        widget = {
            'id': 'widget-1',
            'name': 'test_widget',
            'container': {
                "id": 1
            }
        };

        zhLayoutPreferences      = jasmine.createSpyObj('ZhLayoutPreferences',      ['updateLayouts', 'addIsDefaultLayoutToLayouts', 'applyLayout', 'applyOriginalLayout', 'registerLayoutDataSource']);
        zhPreferences            = jasmine.createSpyObj('ZhPreferences',            ['getUserID', 'validateWidget', 'getDefaultWidgetPreferences']);
        zhLayoutConfigController = jasmine.createSpyObj('ZhLayoutConfigController', ['getConfigWidget']);

        applicationContext = {
            ZhLayoutPreferences: zhLayoutPreferences,
            ZhPreferences: zhPreferences,
            ZhLayoutConfigController: zhLayoutConfigController
        };

        scope = {
            'widget': widget,
            '$on': function(event, fn) {
                if (event == '$destroy') {
                    scopeDestroyFunction = fn;
                }
            }
        };

        query           = jasmine.createSpyObj('query',           ['build']);
        screenService   = jasmine.createSpyObj('screenService',   ['openPopup']);
        metaDataFactory = jasmine.createSpyObj('metaDataFactory', ['factoryDataSource']);

        zhLayoutPreferences.updateLayouts.and.callFake(function(){
            return Q.when();
        });

        zhPreferences.validateWidget.and.callFake(function(){
            return true;
        });

        zhPreferences.getDefaultWidgetPreferences = function(){
            return Q.when([]);
        };

        metaDataFactory.factoryDataSource.and.callFake(function(value){
            return {
                'afterLoadDataSource': function(){},
                'load': function(){}
            };
        });
    });

    afterEach(function() {
        if (scopeDestroyFunction) scopeDestroyFunction();
        scopeDestroyFunction = undefined;
    });

    describe('constructor', function(){

        it('should create customizationCard', function(){
            zhLayoutFloatingCardController = new ZhLayoutFloatingCardController(scope, applicationContext, query, screenService, metaDataFactory);

            var dataSource = {
                'name': 'zh-layout-preferences#/layout',
                'localStorage': false,
                'lazyLoad': true,
                'rest': true,
                'data': []
            };

            expect(metaDataFactory.factoryDataSource).toHaveBeenCalledWith(dataSource);

            expect(scope.customizationCard).toEqual({
                'label': 'Layouts',
                'type': 'layout',
                'openLayoutConfiguration': jasmine.any(Function),
                'refreshLayouts': jasmine.any(Function),
                'applyOriginalLayout': jasmine.any(Function),
                'applyLayout': jasmine.any(Function),
                'dataSource': {
                    'afterLoadDataSource': jasmine.any(Function),
                    'load': jasmine.any(Function)
                }
            });
        });

        it('should add listener on the dataSource to update the DEFAULT_LAYOUT property after load', function(done){
            var dataSource = {
                'data': [{'ID': 1}, {'ID': 2}, {'ID': 3}],
                'afterLoadDataSource': jasmine.createSpy('afterLoadDataSource'),
                'load': function(){}
            };

            var updatedDataSource = [
                {'ID': 1, 'DEFAULT_LAYOUT': false},
                {'ID': 2, 'DEFAULT_LAYOUT': false},
                {'ID': 3, 'DEFAULT_LAYOUT': false}
            ];

            zhLayoutPreferences.addIsDefaultLayoutToLayouts.and.callFake(function(){
                return Q.when(updatedDataSource);
            });

            dataSource.afterLoadDataSource.and.callFake(function(cbk){
                cbk().then(function(){
                    expect(zhLayoutPreferences.addIsDefaultLayoutToLayouts).toHaveBeenCalledWith([{'ID': 1}, {'ID': 2}, {'ID': 3}], widget.id);
                    expect(dataSource.data).toEqual(updatedDataSource);
                    done();
                });
            });

            metaDataFactory.factoryDataSource.and.callFake(function(){
                return dataSource;
            });

            zhLayoutFloatingCardController = new ZhLayoutFloatingCardController(scope, applicationContext, query, screenService, metaDataFactory);

            expect(dataSource.afterLoadDataSource).toHaveBeenCalledWith(jasmine.any(Function));
        });

        it('should add method setAppliedLayout to widget', function(){
            expect(widget.setAppliedLayout).toEqual(undefined);

            zhLayoutFloatingCardController = new ZhLayoutFloatingCardController(scope, applicationContext, query, screenService, metaDataFactory);

            var appliedLayout = {
                'ID': 'applied layout',
                'LABEL_TO_PREFERENCE': 'applied layout'
            };

            expect(widget.setAppliedLayout).toEqual(jasmine.any(Function));

            expect(scope.appliedPreference).toEqual(null);
            widget.setAppliedLayout(appliedLayout);
            expect(scope.appliedPreference).toEqual(appliedLayout);
        });

        it("shouldn't updateLayout when widget is invalid", function() {
            zhPreferences.validateWidget.and.callFake(function(){
                return false;
            });

            zhLayoutFloatingCardController = new ZhLayoutFloatingCardController(scope, applicationContext, query, screenService, metaDataFactory);

            expect(zhLayoutPreferences.updateLayouts).not.toHaveBeenCalled();
        });

        it('should change appliedPreference when call widget.setAppliedLayout', function(){
            var layout = {
                'name': 'Layout 1'
            };

            expect(scope.appliedPreference).toBe(undefined);

            zhLayoutFloatingCardController = new ZhLayoutFloatingCardController(scope, applicationContext, query, screenService, metaDataFactory);

            widget.setAppliedLayout(layout);

            expect(scope.appliedPreference).toBe(layout);
        });

        it('should apply the default preference if widget has it', function(done){
            zhPreferences.getDefaultWidgetPreferences = function(){
                var response = {
                    defaultLayout: {'ID': 2, 'LABEL_TO_PREFERENCE': 'Default'}
                };
                return Q.when(response);
            };

            zhLayoutPreferences.applyLayout.and.callFake(function(){
                expect(zhLayoutPreferences.applyLayout).toHaveBeenCalledWith(widget, {'ID': 2, 'LABEL_TO_PREFERENCE': 'Default'});
                done();
            });

            zhLayoutFloatingCardController = new ZhLayoutFloatingCardController(scope, applicationContext, query, screenService, metaDataFactory);
        });

        it("should apply original layout if widget hasn't default", function(done){
            metaDataFactory.factoryDataSource.and.callFake(function(value){
                return {
                    'afterLoadDataSource': function(){},
                    'load': function(){},
                    'data': [
                        {'ID': 1, 'DEFAULT_LAYOUT': false},
                        {'ID': 2, 'DEFAULT_LAYOUT': false, 'LABEL_TO_PREFERENCE': 'Default'},
                        {'ID': 3, 'DEFAULT_LAYOUT': false}
                    ]
                };
            });

            zhLayoutPreferences.applyOriginalLayout.and.callFake(function(){
                expect(zhLayoutPreferences.applyOriginalLayout).toHaveBeenCalledWith(widget);
                done();
            });

            zhLayoutFloatingCardController = new ZhLayoutFloatingCardController(scope, applicationContext, query, screenService, metaDataFactory);
        });

        it('should not try to apply the default preference if widget does not have it', function(){
            metaDataFactory.factoryDataSource.and.callFake(function(value){
                return {
                    'afterLoadDataSource': function(){},
                    'load': function(){},
                    'data': [
                        {'ID': 1, 'DEFAULT_LAYOUT': false},
                        {'ID': 2, 'DEFAULT_LAYOUT': false},
                        {'ID': 3, 'DEFAULT_LAYOUT': false}
                    ]
                };
            });

            zhLayoutPreferences.updateLayouts.and.callFake(function(){
                return {
                    then: function(cbk){
                        cbk();
                    }
                };
            });

            zhLayoutFloatingCardController = new ZhLayoutFloatingCardController(scope, applicationContext, query, screenService, metaDataFactory);
            expect(zhLayoutPreferences.applyLayout).not.toHaveBeenCalled();
        });

        it('should apply previously applied layout', function(){
            widget.appliedLayout  = { ID: '1', LABEL_TO_PREFERENCE: 'Applied Layout' };
            zhLayoutFloatingCardController = new ZhLayoutFloatingCardController(scope, applicationContext, query, screenService, metaDataFactory);
            expect(zhLayoutPreferences.applyLayout).toHaveBeenCalledWith(widget, { ID: '1', LABEL_TO_PREFERENCE: 'Applied Layout' });
        });
    });

    it('openLayoutConfiguration', function(done){
        zhLayoutFloatingCardController = new ZhLayoutFloatingCardController(scope, applicationContext, query, screenService, metaDataFactory);

        var configWidget = {
            'id': 'layoutConfigWidget'
        };

        zhLayoutConfigController.getConfigWidget.and.callFake(function(){
            return Q.when(configWidget);
        });

        scope.customizationCard.openLayoutConfiguration().then(function(){
            expect(zhLayoutConfigController.getConfigWidget).toHaveBeenCalledWith(widget, {afterLoadDataSource: jasmine.any(Function), load: jasmine.any(Function)});
            expect(screenService.openPopup).toHaveBeenCalledWith(configWidget);
            done();
        });
    });

    it('refreshLayouts', function(done){
        var userID = 'user-1';
        zhLayoutFloatingCardController = new ZhLayoutFloatingCardController(scope, applicationContext, query, screenService, metaDataFactory);

        zhPreferences.getUserID.and.callFake(function(){
            return Q.when(userID);
        });

        var conditions = 'mocked query';

        var getConditions = jasmine.createSpy('where').and.callFake(function(){
            return conditions;
        });

        var equalsMethod = jasmine.createSpy('equals').and.callFake(function(){
            return {'where': getConditions};
        });

        var whereMethod = jasmine.createSpy('where').and.callFake(function(){
            return {'equals': equalsMethod};
        });

        query.build.and.callFake(function(){
            return {'where': whereMethod};
        });

        spyOn(scope.customizationCard.dataSource, 'load');

        scope.customizationCard.refreshLayouts().then(function(){
            expect(scope.customizationCard.dataSource.load).toHaveBeenCalledWith(conditions);
            expect(whereMethod).toHaveBeenCalledWith('WIDGET_ID');
            expect(equalsMethod).toHaveBeenCalledWith(widget.id);
            done();
        });
    });

    it('applyOriginalLayout', function(done){
        zhLayoutFloatingCardController = new ZhLayoutFloatingCardController(scope, applicationContext, query, screenService, metaDataFactory);

        zhLayoutPreferences.applyOriginalLayout.and.callFake(function(){
            return Q.when();
        });

        scope.customizationCard.applyOriginalLayout().then(function(){
            expect(zhLayoutPreferences.applyOriginalLayout).toHaveBeenCalledWith(widget);
            done();
        });
    });

    it('applyPreference', function(done){
        zhLayoutFloatingCardController = new ZhLayoutFloatingCardController(scope, applicationContext, query, screenService, metaDataFactory);

        zhLayoutPreferences.applyLayout.and.callFake(function(){
            return Q.when();
        });

        var layout = {'ID': 1};

        scope.customizationCard.applyLayout(layout).then(function(){
            expect(zhLayoutPreferences.applyLayout).toHaveBeenCalledWith(widget, layout);
            done();
        });
    });

});